import time
import random
from telethon import TelegramClient, events, sync, types
from telethon.tl.functions.messages import ImportChatInviteRequest, CheckChatInviteRequest, GetPeerDialogsRequest, SetTypingRequest
from telethon.tl.functions.channels import JoinChannelRequest, LeaveChannelRequest, GetParticipantRequest
from telethon.errors.rpcerrorlist import ChannelsTooMuchError


class TeleMod:
	
	def __init__(self, p_session, api_id=None, api_hash=None, device_model=None, system_version=None, app_version=None, lang_code=None, system_lang_code=None):
		config = {
			"api_id": api_id or 1339492,
			"api_hash": api_hash or "262d46fcf6756c14123ee6910c6ee5f3"
		}
		self.client = TelegramClient(p_session, **config)
	
	async def connect(self):
		await self.client.connect()
		if not await self.client.is_user_authorized():
			self.disconnect()
			return False
		return True
		 
	async def disconnect(self):
		await self.client.disconnect()
	
	async def send_message(self, username, msgs, mode=0, delay=0.25):
		if mode == 0:
			if isinstance(msgs, (list, tuple)):
				for msg in msgs:
					if msg.replace(" ", "") == "":
						continue
					msg = await self.client.send_message(username, str(msg))
					await self.client(SetTypingRequest(peer=username, action=types.SendMessageTypingAction()))
					await self.client.send_read_acknowledge(username, msg)
					await self.client(SetTypingRequest(peer=username, action=types.SendMessageCancelAction()))
					time.sleep(delay)
			elif isinstance(msgs, (str, int)):
				if str(msgs).replace(" ", "") == "":
					return False
				msgs = await self.client.send_message(username, str(msgs))
				await self.client(SetTypingRequest(peer=username, action=types.SendMessageTypingAction()))
				await self.client.send_read_acknowledge(username, msgs)
				await self.client(SetTypingRequest(peer=username, action=types.SendMessageCancelAction()))
				time.sleep(delay)
			else:
				return False
		else:
			if isinstance(msgs, str):
				msgs = open(msgs, "r").read().split("\n")
				for msg in msgs:
					await self.client.send_message(username, msg)
					time.sleep(delay)	
			else:
				return False
		return True
	
	async def join_group(self, urls, mode=0, delay=0.25):
		if mode == 0:
			if isinstance(urls, (list, tuple)):
				for url in urls:
					if url.replace(" ", "") == "":
						continue
					if "+" in url:
						url = url.split("/")[-1][1:]
						chat_object = await self.client(CheckChatInviteRequest(url))
						if isinstance(chat_object, types.ChatInviteAlready):
							continue
						try:
							await self.client(ImportChatInviteRequest(url))
						except ChannelsTooMuchError:
							self.delete_dialog(1)
							await self.client(ImportChatInviteRequest(url))
						time.sleep(delay)
					else:
						try:
							
							await self.client(JoinChannelRequest(url))
						except ChannelsTooMuchError:
							self.delete_dialog(1)
							await self.client(JoinChannelRequest(url))
						time.sleep(delay)
				return False
			elif isinstance(urls, str):
				if urls.replace(" ", "") == "":
					return False
				if "+" in urls:
					urls = urls.split("/")[-1][1:]
					chat_object = await self.client(CheckChatInviteRequest(urls))
					if isinstance(chat_object, types.ChatInviteAlready):
						return True
					try:
						await self.client(ImportChatInviteRequest(urls))
					except ChannelsTooMuchError:
						self.delete_dialog(1)
						await self.client(ImportChatInviteRequest(urls))
					time.sleep(delay)
				else:
					try:
						await self.client(JoinChannelRequest(urls))
					except ChannelsTooMuchError:
						self.delete_dialog(1)
						await self.client(JoinChannelRequest(urls))
					time.sleep(delay)
			else:
				return False
		else:
			if isinstance(urls, str):
				urls = open(urls, "r").read().split("\n")
				for url in urls:
					self.join_group(url)
			else:
				return False
		return True
	
	async def leave_group(self, urls, mode=0, delay=0.25):
		if mode == 0:
			if isinstance(urls, (list, tuple)):
				for url in urls:
					if url.replace(" ", "") == "":
						continue
					if "+" in url:
						url = url.split("/")[-1][1:]
						chat_object = await self.client(CheckChatInviteRequest(url))
						if isinstance(chat_object, types.ChatInvite):
							continue
						chat_id = chat_object.chat.id
						await self.client.delete_dialog(chat_id)
						time.sleep(delay)
					else:
						await self.client(LeaveChannelRequest(url))
						time.sleep(delay)
			elif isinstance(urls, str):
				if urls.replace(" ", "") == "":
					return False
				if "+" in urls:
					urls = urls.split("/")[-1][1:]
					chat_object = await self.client(CheckChatInviteRequest(urls))
					if isinstance(chat_object, types.ChatInvite):
						return True
					chat_id = chat_object.chat.id
					await self.client.delete_dialog(chat_id)
					time.sleep(delay)
				else:
					await self.client(LeaveChannelRequest(urls))
					time.sleep(delay)
			else:
				return False
		else:
			if isinstance(urls, str):
				urls = open(urls, "r").read().split("\n")
				for url in urls:
					self.leave_group(url)
			else:
				return False
		return True
	
	async def get_peers_dialog(self, peers):
		if isinstance(peers, (list, tuple)):
			result = []
			for peer in peers:
				if peer.replace(" ", "") == "":
					continue
				if "+" in peer:
					peer = peer.split("/")[-1][1:]
					peer = await self.client(CheckChatInviteRequest(peer))
					if isinstance(peer, types.ChatInvite):
						result.append(peer)
						continue
					peer = peer.chat.id
				result.append(await self.client(GetPeerDialogsRequest([peer])))
			return result
		elif isinstance(peers, str):
			if peers.replace(" ", "") == "":
				return False
			if "+" in peers:
				peers = peers.split("/")[-1][1:]
				peers = await self.client(CheckChatInviteRequest(peers))
				if isinstance(peers, types.ChatInvite):
					return peers
				peers = peers.chat.id
			return await self.client(GetPeerDialogsRequest([peers]))
		else:
			return await self.client(GetPeerDialogsRequest([peers]))
	
	async def delete_dialog(self, count=1, write_list=[]):
		deleted = 0
		for dialog in await self.client.iter_dialogs():
			if deleted >= count:
				break
			if dialog.is_group or dialog.is_channel and dialog.id not in write_list:
				try:
					dialog.delete()
					deleted += 1
				except:
					continue
	
	async def get_peers_chat(self, username, filter=None):
		result = []
		if "+" in username:
			username = username.split("/")[-1][1:]
			username = await self.client(CheckChatInviteRequest(username))
			if isinstance(username, types.ChatInvite):
				return result
			username = username.chat.id
		for peer in await self.client.iter_participants(username, filter=filter):
			result.append(peer)
		return result